% ============================================= %
% Description
% ============================================= %

% Produces Figure 3 in the main text, and Table A23 in the Online
% Appendix.

% ============================================= %
% Clear
% ============================================= %
clc;
close all;
clear; 
tic

% ============================================= %
% Simulation options
% ============================================= %

T                 = 33600;     % Number of periods [33600]
N                 = 30;        % Number of countries [30]
HC                = N/2;       % Index of home country
maxhorizon        = 360;       % Horizon max (in months) for term structure
                         
initialize_shocks = 0;         % 0 = use saved shocks
                               % 1 = draw random shocks
Number_portfolios = 6; 

% Sorting variable
sorting_variable = 1;   % 1 = current nominal interest rates or slope of the yield curve
                        % 2 = average nominal interest rates or average slope of the yield curve
                                   
sorting_levelorslope = 1; % 1 = level
                          % 2 = slope
                               
if ismac
    newpath   =  strcat(pwd,'/');
else
    newpath   =  strcat(pwd,'\');
end


% ============================================= %
% Model Parameters
% ============================================= %

% Benchmark
alpha    =  0.0076;
chi      =  0.8895;
gamma    =  0.0413;
tau      =  0.0558;
delta_HC =  0.3569;
delta_u  =  0.4859;
delta_l  =  0.2201;
delta    =  linspace(delta_l, delta_u, N)';
kappa    =  2.7780;
phi      =  0.9107;
theta    =  0.0077;
sigma    =  0.0068;
phi_w    =  0.9944;
theta_w  =  0.0209;
sigma_w  =  0.0028;
pi0      =  -0.0031;
eta_w    =  0.2550;
sigma_pi =  0.0037;

    
% ============================================= %
% Report parameters
% ============================================= %
disp(' ');
disp('Calibration parameters ');
Parameters = [alpha*100 chi tau gamma kappa nan;...
              phi theta*1000 sigma*100 phi_w theta_w*1000 sigma_w*100 ;
              eta_w pi0*100 sigma_pi*100  nan delta(1) delta(end)];
disp('alpha(%) -- chi -- tau -- gamma --kappa -- delta // phi theta(x1000) sigm (%) phi theta(x1000) sigm (%) // eta_w pi0(%) sigma_pi(%) ...  delta(1) -- delta(end)');
latex(Parameters,'%6.2f');
disp(' ');
disp(['delta: ' num2str(delta(HC))]);


% ============================================= %
% Simulate world volatility process
% ============================================= %
if initialize_shocks == 1
    ZW_shocks  = randn(T,2);
    save('ZW_shocks.mat','ZW_shocks'); 
else
    load('ZW_shocks.mat','ZW_shocks'); 
end

ZW         = zeros(T,1);
ZW(1,1)    = theta_w; % Start at the mean volatility
for t=2:T
    ZW(t,1)=(1-phi_w)*theta_w+phi_w*ZW(t-1,1)-sqrt(ZW(t-1,1))*sigma_w*ZW_shocks(t,1); % Law of motion of world standard deviation
end


% ============================================= %
% Simulate idiosyncratic volatility process
% ============================================= %
if initialize_shocks == 1
    Z_shocks   = randn(T,N);
    save('Z_shocks.mat','Z_shocks'); 
else
    load('Z_shocks.mat','Z_shocks'); 
end

Z          = zeros(T,N);
Z(1,:)     = theta; % Start at the mean volatility
for n=1:N
    for t=2:T
        Z(t,n) =(1-phi)*theta+phi*Z(t-1,n)-sqrt(Z(t-1,n))*sigma*Z_shocks(t,n); % Law of motion of world standard deviation
    end
end


% ============================================= %
% Simulate inflation rates
% ============================================= %
if initialize_shocks == 1
    Inflation_shocks   = randn(T,n);
    save('Inflation_shocks.mat','Inflation_shocks'); 
else
    load('Inflation_shocks.mat','Inflation_shocks'); 
end

Inflation    = zeros(T,N);
for n=1:N
    for t=2:T
        Inflation(t,n)    = pi0 + eta_w*ZW(t-1,1)+sigma_pi*Inflation_shocks(t,n);
    end
end


% ============================================= %
% Simulate countries log SDF
% ============================================= %
Log_SDF     = zeros(T,N);
Log_SDF_nom = zeros(T,N);
vol_Log_SDF = zeros(T,N);

for n=1:N
    Log_SDF(1,n)     = - alpha - chi*theta - tau*theta_w; % Initial value of log SDF (shocks  = 0, z processes = their means)
    Log_SDF_nom(1,n) = Log_SDF(1,n)-Inflation(1,n); 
end
for t=2:T
    for n=1:N
        Log_SDF(t,n)     = -(alpha + chi*Z(t-1,n)+ sqrt(gamma*Z(t-1,n))*Z_shocks(t,n)+tau*ZW(t-1,1) + sqrt(delta(n,1)*ZW(t-1,1))*ZW_shocks(t,1) ...
                                + sqrt(kappa*Z(t-1,n))*ZW_shocks(t,2)); % Law of motion of log SDFs
                            
        Log_SDF_nom(t,n) = Log_SDF(t,n)-Inflation(t,n);
               
    end
end


% ============================================= %
% Interest rates and exchange rates
% ============================================= %
Q_ch_nom    = NaN*zeros(T,N);
Q_ch        = NaN*zeros(T,N);
Rf          = NaN*zeros(T,N);
Rf_nom      = NaN*zeros(T,N);
for t=1:T
    for n=1:N
        Rf(t,n)      = alpha + (chi-0.5*gamma-.5*kappa)*Z(t,n)+(tau-0.5*delta(n,1))*ZW(t,1);
        Rf_nom(t,n)  = Rf(t,n) + pi0 + eta_w*ZW(t,1)-0.5*sigma_pi^2;
        if n~=HC
            Q_ch(t,n)       = Log_SDF(t,HC)-Log_SDF(t,n);                       % Country # HC is the home country
            Q_ch_nom(t,n)   = Q_ch(t,n)+Inflation(t,n)-Inflation(t,HC);         
       end
    end
end

TBdif     = zeros(T,N);
for t=1:T
    for n=1:N
        TBdif(t,n)  = Rf(t,n)-Rf(t,HC);           
    end
end

TBdif_lag      = zeros(T,N);
TBdif_nom_lag  = zeros(T,N);
for t=2:T
    for n=1:N
        TBdif_lag(t,n)      = Rf(t-1,n)-Rf(t-1,HC);           
        TBdif_nom_lag(t,n)  = Rf_nom(t-1,n)-Rf_nom(t-1,HC);           
    end
end

Rf_World_Real = nanmean(Rf,2);
Rf_World_Nom  = nanmean(Rf_nom,2);


% Define excess returns
ER     = TBdif_lag-Q_ch;
ER_nom = TBdif_nom_lag-Q_ch_nom;


% ============================================= %
% Yield curves
% ============================================= %

A     = zeros(maxhorizon,N);
B     = zeros(maxhorizon,N);
C     = zeros(maxhorizon,N);
A_nom = zeros(maxhorizon,N);
B_nom = zeros(maxhorizon,N);
C_nom = zeros(maxhorizon,N);

% real 
A(1,:)   = repmat(alpha,1,N) ;
B(1,:)   = chi-.5*(gamma+kappa);
C(1,:)   = tau-.5*delta;

% nominal
A_nom(1,:) = repmat(alpha +pi0 -.5*sigma_pi^2,1,N);
B_nom(1,:) = chi-.5*(gamma+kappa);
C_nom(1,:) = eta_w+tau-.5*delta;

for i=2:maxhorizon
    for n=1:N
        A(i,n)  = alpha+A(i-1,n)+B(i-1,n)*(1-phi)*theta +C(i-1,n)*(1-phi_w)*theta_w;
        B(i,n)  = chi-0.5*(gamma+kappa)+B(i-1,n)*(phi + sigma *sqrt(gamma))-0.5*(B(i-1,n)*sigma )^2;
        C(i,n)  = tau-0.5*delta(n,1) + C(i-1,n)*(phi_w+ sigma_w*sqrt(delta(n,1)))-0.5*(C(i-1,n).*sigma_w)^2;

        A_nom(i,n)  = alpha + pi0 +A_nom(i-1,n)+B_nom(i-1,n)*(1-phi )*theta +C_nom(i-1,n)*(1-phi_w)*theta_w-0.5*sigma_pi^2;
        B_nom(i,n)  = chi-0.5*(gamma+kappa)+B_nom(i-1,n)*(phi + sigma *sqrt(gamma))-0.5*(B_nom(i-1,n)*sigma )^2;
        C_nom(i,n)  = eta_w+tau-0.5*delta(n,1)+C_nom(i-1,n)*(phi_w + sigma_w*sqrt(delta(n,1)))-0.5*(C_nom(i-1,n)*sigma_w)^2;
    end
end   

parpool local
yield      = zeros(T,maxhorizon,N);
yield_nom  = zeros(T,maxhorizon,N);
parfor n=1:N
    for i=1:maxhorizon
        for t=1:T
            yield(t,i,n)     = (A(i,n) + B(i,n)*Z(t,n) + C(i,n)*ZW(t,1))/i; %#ok<PFBNS>
            yield_nom(t,i,n) = (A_nom(i,n) + B_nom(i,n)*Z(t,n) + C_nom(i,n)*ZW(t,1))/i; 
        end
    end
end
delete(gcp)

yieldslopelag      = NaN*zeros(T,maxhorizon,N);
yieldslopelag_nom  = NaN*zeros(T,maxhorizon,N);
for n=1:N
    for i=4:maxhorizon
        for t=2:T
            yieldslopelag(t,i,n)     = yield(t-1,i,n) - yield(t-1,3,n); % Compare yields to the 3-month rate
            yieldslopelag_nom(t,i,n) = yield_nom(t-1,i,n) - yield_nom(t-1,3,n); 
        end
    end
end

% Average yield curves
yield_mean     = (A + B*theta + C*theta_w)./repmat((1:maxhorizon)',1,N);
yield_nom_mean = (A_nom + B_nom*theta + C_nom*theta_w)./repmat((1:maxhorizon)',1,N);

figure('Name','Average real yield curve (closed form)')
plot((1:maxhorizon)/12, yield_mean*1200)
title('Average real yields')
xlabel('Maturity (years)')

figure('Name','Average nominal yield curve (closed form)')
plot((1:maxhorizon)/12, yield_nom_mean*1200)
title('Average nominal yields')
xlabel('Maturity (years)')

%Check mean:
figure('Name','Average simulated real yield curve')
for n=1:N
    plot((1:maxhorizon)/12, mean(yield(:,:,n))*1200);hold on;
end
title('Average real yields')
xlabel('Maturity (years)')

figure('Name','Average simulated nominal yield curve')
for n=1:N
    plot((1:maxhorizon)/12, mean(yield_nom(:,:,n))*1200);hold on;
end
title('Average nominal yields')
xlabel('Maturity (years)')


% Bond returns
bondxreturns      = NaN*zeros(T,maxhorizon,N);
bondxreturns_nom  = NaN*zeros(T,maxhorizon,N);
for n=1:N
    
    % For i=1
    for t=1:T-1
        bondxreturns(t+1,1,n)    =  0;
        bondxreturns_nom(t+1,1,n)=  0;
    end

    for i=2:maxhorizon
        for t=1:T-1
            bondxreturns(t+1,i,n)    =  i*yield(t,i,n) -(i-1)*yield(t+1,i-1,n) - yield(t,1,n);
            bondxreturns_nom(t+1,i,n)=  i*yield_nom(t,i,n) -(i-1)*yield_nom(t+1,i-1,n)-yield_nom(t,1,n);
        end
    end
end

figure('Name','Average simulated real bond excess returns in local currency')
for n=1:N
    plot((1:maxhorizon)/12, nanmean(bondxreturns(:,:,n))*1200);hold on;
end
title('Average real bond excess returns')
xlabel('Maturity (years)')

figure('Name','Average simulated nominal bond excess returns in local currency')
for n=1:N
    plot((1:maxhorizon)/12, nanmean(bondxreturns_nom(:,:,n))*1200);hold on;
end
title('Average nominal bond excess returns')
xlabel('Maturity (years)')

% Keep longest maturity
bondxreturns_LT     = zeros(size(ER));
bondxreturns_nom_LT = zeros(size(ER));
for n=1:N
    bondxreturns_LT(:,n)     = bondxreturns(:,end,n);
    bondxreturns_nom_LT(:,n) = bondxreturns_nom(:,end,n);
end

% Keep 10-year slope
yieldslopelag10y_nom     = zeros(size(ER));
for n=1:N
    yieldslopelag10y_nom(:,n) = yieldslopelag_nom(:,120,n);
end

% Define Bond excess returns
ForeignBondER         = ER + bondxreturns_LT;
ForeignBondER_nom     = ER_nom + bondxreturns_nom_LT;

% Store original currency excess return
ER_nom_orig = ER_nom;


% ============================================= %
% Build Portfolios
% ============================================= %
% Delete first obs
TBdif_nom = TBdif_nom_lag(2:end,:);
TBdif     = TBdif_lag(2:end,:);
Q_ch_nom  = Q_ch_nom(2:end,:);
Q_ch      = Q_ch(2:end,:);
InflTotal = Inflation(2:end,:);
ForeignBondER_nom   = ForeignBondER_nom(2:end,:);
bondxreturns_nom_LT = bondxreturns_nom_LT(2:end,:);
yieldslopelag10y_nom  = yieldslopelag10y_nom(2:end,:);

% Lag interest rate
Rf        = Rf(1:end-1,:);
Rf_US     = Rf(:,HC);
Rf_nom    = Rf_nom(1:end-1,:);
Rf_nom_US = Rf_nom(:,HC);

% Delete HC series
TBdif_nom = [TBdif_nom(:,1:HC-1) TBdif_nom(:,HC+1:end)];
TBdif     = [TBdif(:,1:HC-1) TBdif(:,HC+1:end)];
Q_ch      = [Q_ch(:,1:HC-1) Q_ch(:,HC+1:end)];
Q_ch_nom  = [Q_ch_nom(:,1:HC-1) Q_ch_nom(:,HC+1:end)];
InflTotal = [InflTotal(:,1:HC-1) InflTotal(:,HC+1:end)];
Rf        = [Rf(:,1:HC-1) Rf(:,HC+1:end)];
ER_nom    = TBdif_nom-Q_ch_nom;
ForeignBondER_nom   = [ForeignBondER_nom(:,1:HC-1) ForeignBondER_nom(:,HC+1:end)];
bondxreturns_nom_LT = [bondxreturns_nom_LT(:,1:HC-1) bondxreturns_nom_LT(:,HC+1:end)];
yieldslopelag10y_nom  = [yieldslopelag10y_nom(:,1:HC-1) yieldslopelag10y_nom(:,HC+1:end)];

Delta_C   = delta';
Delta_C   = [Delta_C(:,1:HC-1) Delta_C(:,HC+1:end)];



% ============================================= %
% Returns on Carry Strategies at the short end
% ============================================= %
switch sorting_levelorslope   
    case 1 % sorting on level
         [~,Portfolios_ER,Portfolios_Q,Portfolios_TBdifnom] = Build_portfolios_generic(ER_nom,ER_nom,ER_nom,Q_ch_nom,TBdif_nom,sorting_variable,Number_portfolios); 
        disp(' ');

        Carry_ER   = Portfolios_ER(:,end) - Portfolios_ER(:,1);
        Carry_FD   = Portfolios_TBdifnom(:,end) - Portfolios_TBdifnom(:,1);
       
    case 2 % sorting on slope                
        %  use yield slope instead of short-term interest rate levels to sort variables
        [~,Portfolios_ER,Portfolios_Q,Portfolios_TBdifnom] = Build_portfolios_generic(ER_nom,ER_nom,ER_nom,Q_ch_nom,yieldslopelag10y_nom,sorting_variable,Number_portfolios); 
        disp(' ');

        Carry_ER   =  Portfolios_ER(:,1) - Portfolios_ER(:,end);
        Carry_FD   =  Portfolios_TBdifnom(:,1) - Portfolios_TBdifnom(:,end);

end


Portfolios_ER_ST = Portfolios_ER;

switch sorting_levelorslope 
    case 1
        disp(' ');
        disp('Portfolios of currency excess returns (countries sorted on interest rates)');
        disp(' ');
        disp('Currency Excess returns (mean & std) / Changes in exchange rates (mean & std) / Interest Rate Differences (mean & std)');
    case 2
        disp(' ');
        disp('Portfolios of currency excess returns (countries sorted on slopes)');
        disp(' ');
        disp('Currency Excess returns (mean & std) / Changes in exchange rates (mean & std) / Slope Differences (mean & std)');
end

Results_all = [ nanmean(Portfolios_ER)*12*100; nanstd(Portfolios_ER)*sqrt(12)*100;...
                nanmean(Portfolios_Q)*12*100; nanstd(Portfolios_Q)*sqrt(12)*100;...
                nanmean(Portfolios_TBdifnom)*12*100; nanstd(Portfolios_TBdifnom)*sqrt(12)*100];
latex(Results_all,'%6.2f');

disp('===================');
disp('Returns on Carry Strategies at the short end');
disp('===================');

disp(' ');
disp('HML Carry strategy')
disp(['Mean:               '   num2str(nanmean(Carry_ER)*1200,'%6.2f')]);
disp(['Std:                '   num2str(nanstd(Carry_ER)*sqrt(12)*100,'%6.2f')]);
disp(['Sharpe ratio:       '   num2str(nanmean(Carry_ER)*sqrt(12)/nanstd(Carry_ER),'%6.2f')]);
disp(' ');


% ============================================= %
% Returns on Carry Strategies at the long end
% ============================================= %
ER_nom_LT = ForeignBondER_nom;


switch sorting_levelorslope   
    case 1 % sorting on level
        [~,Portfolios_ER,Portfolios_Q,~] = ...
            Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,Q_ch_nom,TBdif_nom,sorting_variable,Number_portfolios); 

       
    case 2 % sorting on slope                
        %  use yield slope instead of short-term interest rate levels to sort variables
        [~,Portfolios_ER,Portfolios_Q,~] = ...
            Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,Q_ch_nom,yieldslopelag10y_nom,sorting_variable,Number_portfolios); 

end

Portfolios_ER_LT = Portfolios_ER;
Portfolios_Q_LT  = Portfolios_Q;

ER_nom_LT = bondxreturns_nom_LT;

switch sorting_levelorslope   
    case 1 % sorting on level
        [~,Portfolios_ER,Portfolios_Q,Portfolios_TBdifnom] = ...
            Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,Q_ch_nom,TBdif_nom,sorting_variable,Number_portfolios); 

        %  TEMP: to look at slopes when sorting on leves
        [~,~,Portfolios_IR_slopes,Portfolios_IR_levels] = ...
            Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,yieldslopelag10y_nom,TBdif_nom,sorting_variable,Number_portfolios); 
        disp('Interest Rate Differences / Slope  Across Portfolios');
        Results_all = [ nanmean(Portfolios_IR_levels)*12*100; ...
                        nanmean(Portfolios_IR_slopes)*12*100];
        latex(Results_all,'%6.2f');
       
    case 2 % sorting on slope                
        %  use yield slope instead of short-term interest rate levels to sort variables
        [~,Portfolios_ER,Portfolios_Q,Portfolios_TBdifnom] = ...
            Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,Q_ch_nom,yieldslopelag10y_nom,sorting_variable,Number_portfolios); 
        
        %  TEMP: to look at levels when sorting on slopes
        [~,~,Portfolios_IR_levels,Portfolios_IR_slopes] = ...
            Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,TBdif_nom,yieldslopelag10y_nom,sorting_variable,Number_portfolios); 
        disp('Interest Rate Differences / Slope  Across Portfolios');
        Results_all = [ nanmean(Portfolios_IR_levels)*12*100; ...
                        nanmean(Portfolios_IR_slopes)*12*100];
        latex(Results_all,'%6.2f');

end

% -----------------------------------------
% TEMP: TO DELETE AFTER STUDYING PREDICTABILITY AT THE COUNTRY LEVEL
% y = bondxreturns_nom_LT(:,1);
% x = yieldslopelag10y_nom(:,1);
% b = regress(y,[ones(size(y,1),1) x]);

% y = bondxreturns_nom_LT(:,end);
% x = yieldslopelag10y_nom(:,end);
% b = regress(y,[ones(size(y,1),1) x])

% y = bondxreturns_nom_LT(:,1);
% x = TBdif_nom(:,1); %NB: This uses the interest rate difference across countries
% b = regress(y,[ones(size(y,1),1) x])

% y = bondxreturns_nom_LT(:,end);
% x = TBdif_nom(:,end); %NB: This uses the interest rate difference across countries
% b = regress(y,[ones(size(y,1),1) x])

% corr(yield(2:end-1,3,1),bondxreturns(3:end,120,1))
% corr(yield(2:end-1,3,end),bondxreturns(3:end,120,end))


% % first country, at the 10-year horizon (120 months), predictability using the level of the short-term interest rate
% 
% y = bondxreturns(3:end,120,1);
% x = yield(2:end-1,3,1); 
% [b,bint] = regress(y,[ones(size(y,1),1) x]);
% disp('Slope coefficient in a regression of 10-year bond excess return on the level of the short-term interest rate (case of first country, low delta, all real variables):');
% disp(b(2));
% disp(bint(2,:));
% 
% % last country, at the 10-year horizon (120 months), predictability using the level of the short-term interest rate
% y = bondxreturns(3:end,120,end);
% x = yield(2:end-1,3,end); 
% [b,bint] = regress(y,[ones(size(y,1),1) x]);
% disp('Slope coefficient in a regression of 10-year bond excess return on the level of the short-term interest rate (case of last country, high delta, all real variables):');
% disp(b(2));
% disp(bint(2,:));
% 
% % first country, at the 10-year horizon (120 months), predictability using the slope of the yield curve
% y = bondxreturns(3:end,120,1);
% x = yield(2:end-1,120,1)-yield(2:end-1,3,1); 
% [b,bint] = regress(y,[ones(size(y,1),1) x]);
% disp('Slope coefficient in a regression of 10-year bond excess return on the slope of the yield curve (case of first country, low delta, all real variables):');
% disp(b(2));
% disp(bint(2,:));
% 
% % last country, at the 10-year horizon (120 months), predictability using the slope of the yield curve
% y = bondxreturns(3:end,120,end);
% x = yield(2:end-1,120,end)-yield(2:end-1,3,end); 
% [b,bint] = regress(y,[ones(size(y,1),1) x]);
% disp('Slope coefficient in a regression of 10-year bond excess return on the slope of the yield curve (case of last country, high delta, all real variables):');
% disp(b(2));
% disp(bint(2,:));

b   = NaN(2,N);
R2  = NaN(N,1);
bint= NaN(2,2,N);
for n =1:N
% all countries, at the 10-year horizon (120 months), predictability using the level of the short-term interest rate
y = bondxreturns(3:end,120,n);
x = yield(2:end-1,3,n); 
[b(:,n),bint(:,:,n),R,RINT,STATS] = regress(y,[ones(size(y,1),1) x]);
R2(n,1) = STATS(1);
end
disp('Slope coefficient in a regression of 10-year bond excess return on the level of the short-term interest rate, for each country:');
disp([b(2,:)',reshape(bint(2,:,:),2,N)',R2]);

b   = NaN(2,N);
R2  = NaN(N,1);
bint= NaN(2,2,N);
for n = 1:N
% all countries, at the 10-year horizon (120 months), predictability using the slope of the yield curve
y = bondxreturns(3:end,120,n);
x = yield(2:end-1,120,n)-yield(2:end-1,3,n); 
[b(:,n),bint(:,:,n),R,RINT,STATS] = regress(y,[ones(size(y,1),1) x]);
R2(n,1) = STATS(1);
end
disp('Slope coefficient in a regression of 10-year bond excess return on the slope of the yield curve, for each country:');
disp([b(2,:)',reshape(bint(2,:,:),2,N)',R2]);

corr_z_xr=NaN(N,1);
corr_z_rf=NaN(N,1);
corr_z_sl=NaN(N,1);
for n = 1:N
corr_z_xr(n,1) = corr(bondxreturns(3:end,120,n),Z(2:end-1,n));
corr_z_rf(n,1) = corr(yield(2:end-1,3,n),Z(2:end-1,n)); 
corr_z_sl(n,1) = corr(yield(2:end-1,120,n)-yield(2:end-1,3,n),Z(2:end-1,n)); 
end
disp('Correlation coefficient of z with bond xr / riskless rate / YC slope, for each country:');
disp([corr_z_xr corr_z_rf corr_z_sl]);

% -----------------------------------------


Portfolios_ER_LT_local = Portfolios_ER;


disp(' ');
disp('Portfolios of bond excess returns (countries sorted on interest rates)');
disp(' ');

disp(['Foreign Bond Excess returnsin USD / Bond XR in local currency /'...
        'Currency XR / Changes in exchange rates / Interest Rate (or slope) Differences']);

Results_all = [ nanmean(Portfolios_ER_LT)*12*100; ...
                nanmean(Portfolios_ER_LT_local)*12*100; ...
                nanmean(Portfolios_ER_ST)*12*100; ...
                nanmean(Portfolios_Q_LT)*12*100; ...
                nanmean(Portfolios_TBdifnom)*12*100];
latex(Results_all,'%6.2f');


switch sorting_levelorslope   
    case 1 % sorting on level
        Carry_ER_LT   = Portfolios_ER_LT(:,end) - Portfolios_ER_LT(:,1);
    case 2 % sorting on slope                
        Carry_ER_LT   =  Portfolios_ER_LT(:,1) - Portfolios_ER_LT(:,end);
end

disp('===================');
disp('Returns on Carry Strategies at the long end');
disp('===================');

disp(' ');
disp('HML Carry strategy')
disp(['Mean:               '   num2str(nanmean(Carry_ER_LT)*1200,'%6.2f')]);
disp(['Std:                '   num2str(nanstd(Carry_ER_LT)*sqrt(12)*100,'%6.2f')]);
disp(['Sharpe ratio:       '   num2str(nanmean(Carry_ER_LT)*sqrt(12)/nanstd(Carry_ER_LT),'%6.2f')]);
disp(' ');


% ============================================= %
% Graph: Term Structure of Currency Carry Trade Risk Premia
% ============================================= %

AverageER      = NaN*zeros(maxhorizon,1);

% For each maturity
for maturity = 1:maxhorizon
    
    bondxreturns_nom_LT = zeros(size(ER));
    for n=1:N
        bondxreturns_nom_LT(:,n) = bondxreturns_nom(:,maturity,n);
    end
    ForeignBondER_nom     = ER_nom_orig + bondxreturns_nom_LT;
    
    % Delete first obs
    ForeignBondER_nom   = ForeignBondER_nom(2:end,:);
    
    % Delete HC series
    ForeignBondER_nom   = [ForeignBondER_nom(:,1:HC-1) ForeignBondER_nom(:,HC+1:end)];
    
    % Build portfolios
    ER_nom_LT = ForeignBondER_nom;
    
    switch sorting_levelorslope   
        case 1 % sorting on level
            [~,Portfolios_ER,Portfolios_Q,~] = ...
                Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,Q_ch_nom,TBdif_nom,sorting_variable,Number_portfolios); 
            
            % Carry Long-Short
            Portfolios_ER_LT = Portfolios_ER;
            Carry_ER_LT   = Portfolios_ER_LT(:,end) - Portfolios_ER_LT(:,1);


        case 2 % sorting on slope                
            %  use yield slope instead of short-term interest rate levels to sort variables
               [~,Portfolios_ER,Portfolios_Q,~] = ...
                Build_portfolios_generic(ER_nom_LT,ER_nom_LT,ER_nom_LT,Q_ch_nom,yieldslopelag10y_nom,sorting_variable,Number_portfolios); 
            
            % Carry Long-Short
            Portfolios_ER_LT = Portfolios_ER;
            Carry_ER_LT   = Portfolios_ER_LT(:,1) - Portfolios_ER_LT(:,end);

    end 

    AverageER(maturity,1) = mean(Carry_ER_LT)*12*100;

end
maturity_scale = 1:1:maxhorizon;
figure('Name','Term Structure of Currency Carry Trade Risk Premia')
plot(maturity_scale/4,AverageER,'lineWidth',6);
ylabel('Dollar Excess Returns')
xlabel('Maturity (in quarters)')
ylim([-8 8]); grid on;
print('-depsc2',strcat('TermStructureCarry_LRV_Sorting_',int2str(sorting_levelorslope),'.eps'));


save(strcat(newpath,'TermStructureGraph_LRV_Sorting_',int2str(sorting_levelorslope),'.mat'),'AverageER');

% ===============
% TEMP MATERIAL
% ==============
maturity_temp = 360;
bondxret_temp = bondxreturns_nom(:,maturity_temp,1);
disp(nanmean(bondxret_temp)*12*100);
bondxret_temp = bondxreturns_nom(:,maturity_temp,end);
disp(nanmean(bondxret_temp)*12*100);